@@ -38,14 +38,9 @@ from Cryptodome.Hash import SHAKE128
 from Cryptodome.PublicKey import ECC
 
 def load_file(filename, mode="rb"):
-    fd = open(pycryptodome_filename([
-                                    "Cryptodome",
-                                    "SelfTest",
-                                    "PublicKey",
-                                    "test_vectors",
-                                    "ECC",
-                                    ], filename), mode)
-    return fd.read()
+    comps = [ "Cryptodome", "SelfTest", "PublicKey", "test_vectors", "ECC" ]
+    with open(pycryptodome_filename(comps, filename), mode) as fd:
+        return fd.read()
 
 
 def compact(lines):
@@ -424,7 +419,7 @@ gVnJp9EBND/tHQ==
         from Cryptodome.PublicKey.ECC import UnsupportedEccFeature
         try:
             ECC.import_key(curve)
-        except UnsupportedEccFeature, uef:
+        except UnsupportedEccFeature as uef:
             assert("1.3.132.0.33" in str(uef))
         else:
             assert(False)
